/* ========================================================================== *\
    Watch Dog Adapter Test Program ---------------- DECISION Taiwan

    This is a sample C program to control WatchDog Adapter. You can
    Borland C, MicroSoft C ... to compiler it.

    Usage :  WatchDog [ IOAddr [ Enable/Disable ] ]

    IOAddr : the base address of WatchDog Adapter
	     legal value : 0x000 to 0xFFE

    Enable/Disable : 1=Enable, 0=Disable WatchDog operation

    Example : WatchDog 0xBBC ( and Adapter JP5 set to 5 )
       Enable WatchDog to check system. System will reboot if
       the you don't press any key more than 1 second.
\* ========================================================================= */

#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <time.h>
#include <math.h>

#include "WatchDog.h"

// Definition in Main.c
BOOL NEAR InitApplication( HANDLE );
HWND NEAR InitInstance( HANDLE, int );
LRESULT FAR PASCAL WndProc( HWND, UINT, WPARAM, LPARAM );
BOOL FAR PASCAL AboutDlgProc( HWND, UINT, WPARAM, LPARAM );
BOOL FAR PASCAL SetupDlgProc( HWND, UINT, WPARAM, LPARAM );
                      
#ifdef _WIN32                      
#define TimeOffset  30
#else
#define TimeOffset  60
#endif

static int   AutoRefresh=1;
static int   WdgAddr=0xBBC;
static int   WdgEnable=0;
static DWORD RefreshTime=4000;
static DWORD LastRefTime; 

#ifdef _WIN32
#define GetMyTime()    timeGetTime()
#else
#define GetMyTime()    (DWORD)clock()*1000/CLOCKS_PER_SEC
#endif

int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow )
{
    MSG    msg ;
    DWORD  NowTime;
    char   szBuffer[30];

    if (!hPrevInstance)
        if (!InitApplication( hInstance )) return ( FALSE ) ;
    if ( !InitInstance( hInstance, nCmdShow )) return ( FALSE ) ;
    while ( WM_QUIT != msg.message ) {
        if ( PeekMessage(&msg, (HWND)NULL, 0, 0, PM_REMOVE) )  {
            TranslateMessage( &msg ) ;
            DispatchMessage( &msg ) ;
        }
        NowTime = GetMyTime();
        if ( ( 1 == AutoRefresh ) && ( 1 == WdgEnable ) ) {
            if ( NowTime - LastRefTime > RefreshTime - TimeOffset ) {
                watchdog_refresh(WdgAddr);
                LastRefTime = GetMyTime();
            }
        }
        if ( WdgEnable ) {
            sprintf(szBuffer, "WatchDog Tester    %ld ms", 
                RefreshTime-NowTime+LastRefTime);
            SetWindowText(msg.hwnd, szBuffer);
        }
    } // end of Msg != WM_QUIT
    return ( (int)msg.wParam );
} // end of WinMain()


BOOL NEAR InitApplication( HANDLE hInstance )
{
   WNDCLASS  wndclass ;
   // register tty window class
   wndclass.style =         0 ;
   wndclass.lpfnWndProc =   WndProc ;
   wndclass.cbClsExtra =    0 ;
   wndclass.cbWndExtra =    sizeof( LONG );
   wndclass.hInstance =     hInstance ;
   wndclass.hIcon =         NULL; /*LoadIcon( (HINSTANCE)NULL, IDI_APPLICATION );*/
   wndclass.hCursor =       LoadCursor( (HINSTANCE)NULL, IDC_ARROW ) ;
   wndclass.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1) ;
   wndclass.lpszMenuName =  MAKEINTRESOURCE( IDR_MAINMENU ) ;
   wndclass.lpszClassName = "WatchDog" ;
   return( RegisterClass( &wndclass ) ) ;
} // end of InitApplication()

HWND NEAR InitInstance( HANDLE hInstance, int nCmdShow )
{
   HWND  hWnd ;
   // create the TTY window
   hWnd = CreateWindow( "WatchDog", "WatchDog Tester", 
       WS_POPUPWINDOW | WS_CAPTION, 
       CW_USEDEFAULT, CW_USEDEFAULT,
       400,  
       GetSystemMetrics(SM_CYCAPTION) + GetSystemMetrics(SM_CYMENU)
       + 2 * GetSystemMetrics(SM_CYBORDER) + 2, 
       (HWND)NULL, (HMENU)NULL, hInstance, (void FAR *)NULL ) ;
   if (hWnd == (HWND)NULL) return ( (HWND)NULL ) ;
   ShowWindow( hWnd, nCmdShow ) ;
   UpdateWindow( hWnd ) ;
   return ( hWnd ) ;
} // end of InitInstance()

LRESULT FAR PASCAL WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    static HMENU  hMenu;
    switch (uMsg)
    {
       case WM_CREATE:
            hMenu = GetMenu( hWnd ) ;
            CheckMenuItem( hMenu, IDR_AUTOREFRESH, MF_CHECKED );
            EnableMenuItem( hMenu, IDR_DISABLE, MF_GRAYED );
            EnableMenuItem( hMenu, IDR_AUTOREFRESH, MF_GRAYED );
            EnableMenuItem( hMenu, IDR_REFRESH, MF_GRAYED );
            return ( (LRESULT) TRUE);
       case WM_COMMAND:
            switch ( LOWORD( wParam ) ) {
            case IDR_ENABLE:
                 WdgEnable = 1;
                 watchdog(WdgAddr);
                 hMenu = GetMenu( hWnd ) ;
                 EnableMenuItem( hMenu, IDR_ENABLE, MF_GRAYED );
                 EnableMenuItem( hMenu, IDR_DISABLE, MF_ENABLED );
                 EnableMenuItem( hMenu, IDR_AUTOREFRESH, MF_ENABLED );
                 EnableMenuItem( hMenu, IDR_REFRESH, MF_ENABLED );
                 if ( AutoRefresh >= 1 ) {
                      CheckMenuItem( hMenu, IDR_AUTOREFRESH, MF_CHECKED );
                      LastRefTime = GetMyTime();
                      AutoRefresh = 1;
                 }
                 else CheckMenuItem( hMenu, IDR_AUTOREFRESH, MF_UNCHECKED ); 
                 break ;
            case IDR_DISABLE:
                 WdgEnable = 0;
                 disable_watchdog(WdgAddr);
                 hMenu = GetMenu( hWnd ) ;
                 EnableMenuItem( hMenu, IDR_ENABLE, MF_ENABLED );
                 EnableMenuItem( hMenu, IDR_DISABLE, MF_GRAYED );
                 EnableMenuItem( hMenu, IDR_AUTOREFRESH, MF_GRAYED );
                 EnableMenuItem( hMenu, IDR_REFRESH, MF_GRAYED );
                 AutoRefresh = 2;
                 break;
            case IDR_AUTOREFRESH:
                 hMenu = GetMenu( hWnd ) ;
                 if ( 0 == AutoRefresh  ) {
                     AutoRefresh = 1;
                     CheckMenuItem( hMenu, IDR_AUTOREFRESH, MF_CHECKED );
                 } else {
                     AutoRefresh = 0;
                     CheckMenuItem( hMenu, IDR_AUTOREFRESH, MF_UNCHECKED );
                 }
                 break;
            case IDR_REFRESH:
                 LastRefTime = GetMyTime();
                 watchdog_refresh(WdgAddr);
                 break;
            case IDR_SETUP:
                 {
                     DLGPROC  lpProcInstance ;
                     lpProcInstance=
                         (DLGPROC)MakeProcInstance((FARPROC)SetupDlgProc,
                                   GETHINST(hWnd) );
                     DialogBoxParam(GETHINST(hWnd), MAKEINTRESOURCE(IDD_SETUP),
                                   hWnd, lpProcInstance, 0L );
                     FreeProcInstance((FARPROC) lpProcInstance);
                 }
                 break;
            case IDD_MAINABOUT:
                 {
                 DLGPROC  lpProcInstance ;
                 lpProcInstance=(DLGPROC)MakeProcInstance((FARPROC)AboutDlgProc,
                                   GETHINST(hWnd) );
                 DialogBoxParam(GETHINST(hWnd), MAKEINTRESOURCE(IDD_MAINABOUT),
                                   hWnd, lpProcInstance, 0L );
                 FreeProcInstance((FARPROC) lpProcInstance);
                 }
                 break;
            case IDR_REMOTE_ON:
                 watchdog_control(WdgAddr, 0);
                 break;
            case IDR_REMOTE_OFF:
                 watchdog_control(WdgAddr, 1);
                 break;
            case IDR_EXIT:
                 PostMessage( hWnd, WM_CLOSE, 0, 0L ) ;
                 break ;
            }
            break;
       case WM_DESTROY:
            disable_watchdog(WdgAddr);
            PostQuitMessage( 0 ) ;
            break ;
       case WM_CLOSE:
            if (IDOK != MessageBox( hWnd, "OK to stop Test?", "WatchDog Tester",
                                   MB_OKCANCEL ))
                break ;
       default:
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) ) ;
    }
    return 0L ;
} // end of WndProc()

BOOL FAR PASCAL AboutDlgProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   switch (uMsg)
   {
      case WM_INITDIALOG: return ( TRUE ) ;
      case WM_COMMAND:
         if (LOWORD( wParam ) == IDD_OK)
         {
            EndDialog( hDlg, TRUE ) ;
            return ( TRUE ) ;
         }
         break;
   }
   return ( FALSE ) ;
} // end of AboutDlgProc()

BOOL FAR PASCAL SetupDlgProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   char     szBuffer[20];
   DWORD    TickTimer;
   int      DefaultTick = 0;
   int      TickCount;
   int      NewWdgAddr;
   int      TickSel;

   switch (uMsg)
   {
      case WM_INITDIALOG: 
           // Initial IOAddress
           wsprintf(szBuffer, "%03x", WdgAddr);
           SetDlgItemText(hDlg, IDC_IOADDR, (LPCSTR) szBuffer);
           // Initial Tick Count Listing
           wsprintf( szBuffer, "  1 -> 62.5ms") ;
           SendDlgItemMessage( hDlg, IDC_TIMER, CB_ADDSTRING, 0, (LPARAM) (LPSTR) szBuffer );
           if ( 62 == RefreshTime ) DefaultTick = 1;
           TickTimer = 125;
           for (TickCount=2; TickCount<=11; TickCount++)
           {
               if ( TickTimer == RefreshTime ) DefaultTick = TickCount;
               if ( TickTimer < 1000 )
                    wsprintf( szBuffer, " %2d -> %3ld ms", TickCount, TickTimer) ;
               else wsprintf( szBuffer, " %2d -> %3ld sec", TickCount, TickTimer/1000) ;
               SendDlgItemMessage( hDlg, IDC_TIMER, CB_ADDSTRING, 0,
                      (LPARAM) (LPSTR) szBuffer ) ;
               TickTimer *= 2;
           }
           SendDlgItemMessage( hDlg, IDC_TIMER, CB_SETCURSEL,
                      (WPARAM)( DefaultTick - 1), 0L ) ;
           return ( TRUE ) ;
      case WM_COMMAND: 
           switch ( LOWORD(wParam) ) 
           {
           case IDD_OK :
                GetDlgItemText(hDlg, IDC_IOADDR, (LPSTR) szBuffer, 20);
                sscanf(szBuffer, "%x", &NewWdgAddr);
                if ( ( NewWdgAddr <= 0 ) || ( NewWdgAddr >= 0xFFF ) )
                {
                    MessageBox(hDlg, "IO Address Error", "Message", MB_OK);
                    return(TRUE);
                } else WdgAddr = NewWdgAddr;
                TickSel = LOWORD( SendDlgItemMessage( hDlg, 
                    IDC_TIMER, CB_GETCURSEL,  0, 0L ) ) ;
                if ( TickSel == 0 ) RefreshTime = 62;
                else RefreshTime = (DWORD)(62.5 * pow(2,TickSel));
                EndDialog( hDlg, TRUE ) ;
                return ( TRUE ) ;
           case IDD_CANCEL :
                EndDialog( hDlg, TRUE ) ;
                return ( TRUE ) ;
           }
           break;
   }
   return ( FALSE ) ;
} // end of AboutDlgProc()
